function [s, d] = MAPsas(sPrior, xiPrior);

%  [s, d] = MAPsas(sigPrior, noisePrior) computes MAP function:
% 
%  s(d) = argmax P(s|d) = argmax P(d|s)P(s) = 
%       = argmax P(d-s)P(s) = argmax P(xi)P(s)
%
%  sPrior and xiPrior are Nx2 matrices, containing the discrete prior PDFs (2nd
%  column) of the signal and noise respectively, and the corresponding bin centers.

sigY = sPrior(:,2);
sigX = sPrior(:,1);
ipssig = sigX(2)-sigX(1);

noiseY = xiPrior(:,2);
noiseX = xiPrior(:,1);
ipsnoise = noiseX(2)-noiseX(1);

% Verify whether the spacing between x axes points corresponding to the
% signal and noise PDFs is respectively the same.
if (abs(ipsnoise - ipssig) > 1e-4)
  error('Signal and noise PDFs should be equally spaced on x axis.');
end

jointProbY = conv2(diag(sigY),noiseY');%whos jointProbY
posteriorProbY = jointProbY ./ (ones(size(jointProbY,1),1)*mean(jointProbY));
%posteriorProbY = jointProbY;
[values,indices] = max(posteriorProbY);
d = [sigX+noiseX(1); sigX(length(sigX)) + noiseX(2:length(noiseX))];
s = sigX(indices);

%% s is only taken into consideration over the support of the signal PDF.
n0sPoss = interp1(noiseX, [1:length(noiseX)]', 0,'linear');
supp = [ceil(n0sPoss):length(d)-(length(noiseX)-floor(n0sPoss))];
d = d(supp);
s = s(supp); 

% plot the MAP estimator compared against the identity function
% figure
% plot(d,s,d,d,'-r');grid;axis square;%pause;

return