function y = main_SaS_dtdwt_global_v4(x)
% Dual-tree DWT Denoising Algorithm using SaS modeling of wavelet
% coefficients
% Usage :
%        y = denoising_dtdwt(x)
% INPUT :
%        x - a noisy image
% OUTPUT :
%        y - the corresponding denoised image

% Number of Stages
J = 5;
I=sqrt(-1);

% symmetric extension
L = length(x); % length of the original image.
N = L+2^J;     % length after extension.
x = symextend(x,2^(J-1));

load nor_dualtree    % run normaliz_coefcalc_dual_tree to generate this mat file.

% Forward dual-tree DWT
% Either FSfarras or AntonB function can be used to compute the stage 1 filters  
%[Faf, Fsf] = FSfarras;
[Faf, Fsf] = AntonB;
[af, sf] = dualfilt1;
W = cplxdual2D(x, J, Faf, af);
W = normcoef(W,J,nor);

% Noise variance estimation using robust median estimator..
tmp = W{1}{1}{1}{3};
Nsig = median(abs(tmp(:)))/0.6745;

for scale = 1:J-1
    for dir = 1:2
        for dir1 = 1:3
            
%            disp(['Processing scale ' num2str(scale) 'dir' num2str(dir) 'dir1' num2str(dir1)]);
            
            % Noisy complex coefficients
            %Real part
            Y_coef_real = W{scale}{1}{dir}{dir1};
            % imaginary part
            Y_coef_imag = W{scale}{2}{dir}{dir1};
            % The corresponding noisy parent coefficients
            %Real part
            Y_parent_real = W{scale+1}{1}{dir}{dir1};
            % imaginary part
            Y_parent_imag = W{scale+1}{2}{dir}{dir1};
            % Extend noisy parent matrix to make the matrix size the same as the coefficient matrix.
            Y_parent_real  = expand(Y_parent_real);
            Y_parent_imag   = expand(Y_parent_imag);
            
            % Estimation of signal dispersion:
            % - Characteristic function estimate of gama:
            [alpha,disp] = SaSGfit_manual(Y_coef_real(:),Nsig);
%            [alpha disp Nsig]
            
            % Compute the fitted model density of the image (SaS)
            % and the model of the noise (Gaussian white).
            o_real = [Y_coef_real(:);Y_parent_real(:)];
            o_imag = [Y_coef_imag(:);Y_parent_imag(:)];
            o = [o_real;o_imag];
            q = max(max(o), abs(min(o)));
            x1 = (linspace(-q,q,101))';
            
            im = stablepdf(x1,alpha,0,disp,0,2);
            imModel = [x1 im];
            
            noise = normpdf(x1,0,Nsig);
            noiseModel = [x1 noise];
            
            %%Compute Bayes least squares coring function:
            [fny,fnx]=makeCoreSaS(imModel, noiseModel);
            %[fny,fnx]=MAPSaS(imModel, noiseModel);
%            figure;plot(fnx,fny);
            
            %Compute the estimates given the lookup table            
            [My,Ny]=size(Y_coef_real);
            Y_coef_real = interp1(fnx, fny, Y_coef_real(:),'*spline');
            Y_coef_imag = interp1(fnx, fny, Y_coef_imag(:),'*spline');
                        
            W{scale}{1}{dir}{dir1} = reshape(Y_coef_real,My,Ny);
            W{scale}{2}{dir}{dir1} = reshape(Y_coef_imag,My,Ny);
            
        end
    end
end

% Inverse Transform
W = unnormcoef(W,J,nor);
y = icplxdual2D(W, J, Fsf, sf);

% Extract the image
ind = 2^(J-1)+1:2^(J-1)+L;
y = y(ind,ind);